package com.huiju.welcome.service.impl;

import com.huiju.welcome.mapper.SysUserMapper;
import com.huiju.welcome.model.SysUser;
import com.huiju.welcome.model.TaCustomer;
import com.huiju.welcome.service.IMessageService;
import com.huiju.welcome.service.ITaMessageRecordService;
import com.huiju.welcome.utils.CommConstant;
import com.huiju.welcome.utils.MiniApp;
import com.huiju.welcome.utils.SMSUtils;
import com.huiju.welcome.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class MessageServiceImpl implements IMessageService {
    @Autowired
    SysUserMapper sysUserMapper;

    @Autowired
    MiniApp miniApp;

    @Autowired
    SMSUtils smsUtils;

    @Autowired
    ITaMessageRecordService iTaMessageRecordService;

    @Override
    public boolean notifyManager(TaCustomer taCustomer) {
        // 置业经理,渠道经理,策划经理
        List<String> identities = new ArrayList<String>();
        
        if (CommConstant.CUSTOMER_OF_CHANNEL.equals(taCustomer.getCustomerType())){
            identities.add(CommConstant.IDENTITY_CHANNEL_MANAGER);
        }
        identities.add(CommConstant.IDENTITY_CONSULTANT_MANAGER);
        identities.add(CommConstant.IDENTITY_PLANNING_MANAGER);
        //identities.add(CommConstant.IDENTITY_GENERAL_MANAGER);

        // 推送给置业经理或者渠道经理 + 渠道总
        List<SysUser> sysUserList = sysUserMapper.filterByIdentities(identities);
        if (null != sysUserList) {
            for (SysUser sysUser: sysUserList) {
                if (StringUtils.isEmpty(sysUser.getOpenid())) {
                    log.error("用户 {} 未绑定小程序", sysUser.getUsername());
                }

                notifyCustomer2User(sysUser.getOpenid(), sysUser.getPhone(), taCustomer);
            }
        }

        return true;
    }

    @Override
    public boolean notifyConsultant(TaCustomer taCustomer) {
        Integer consultantId = taCustomer.getConsultantId();
        if (null == consultantId) {
            return false;
        }

        SysUser sysUser = sysUserMapper.selectById(consultantId);
        if (null == sysUser) {
            return false;
        }

        if (StringUtils.isEmpty(sysUser.getOpenid())) {
            log.error("用户 {} 未绑定小程序", sysUser.getUsername());
        }

        return notifyCustomer2User(sysUser.getOpenid(), sysUser.getPhone(), taCustomer);
    }

    @Override
    public boolean toggledByCustomer(TaCustomer taCustomer) {
        Boolean b1 = notifyConsultant(taCustomer);
        Boolean b2 = notifyManager(taCustomer);

        return b1 || b2;
    }

    private boolean notifyCustomer2User(String openid, String phone, TaCustomer taCustomer) {
        List<Object> data = new ArrayList<Object>(){{
            add(taCustomer.getCustomerName());
            add(taCustomer.getPhone());
            add("您有客户来访, 请及时接待!");
        }};

        // 先尝试发送小程序模板消息
        String tpl = miniApp.getTpls().getVisitor();
        String page = CommConstant.PAGE_MY_CUSTOMER + "?id=" + taCustomer.getCustomerId();

        if (null == openid || miniApp.sendTPLMessage(tpl, openid, page, data).getCode() == -1) {
            // 改发短信
            if (null != phone) {
                boolean needSend = iTaMessageRecordService.saveMessageOrNot(phone, "", "sms", data, false);
                if (!needSend) return true;

                boolean s = smsUtils.sendVisitor(phone, null == taCustomer ? "": StringUtils.ifNull(taCustomer.getCustomerName(), ""));
                iTaMessageRecordService.saveMessageOrNot(phone, "", "sms", data, true);
                return s;
            }
        }

        return true;
    }
}
