package com.huiju.welcome.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.common.Action;
import com.huiju.welcome.common.WebSocketServer;
import com.huiju.welcome.mapper.TaFirstUsherRecordMapper;
import com.huiju.welcome.model.*;
import com.huiju.welcome.service.*;
import com.huiju.welcome.utils.CommConstant;
import com.huiju.welcome.utils.StatusUtils;
import com.huiju.welcome.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-06-04
 */
@Service
public class TaFirstUsherRecordServiceImpl extends ServiceImpl<TaFirstUsherRecordMapper, TaFirstUsherRecord> implements ITaFirstUsherRecordService {
    @Autowired
    TaFirstUsherRecordMapper taFirstUsherRecordMapper;

    @Autowired
    ITaMainUsherRecordService iTaMainUsherRecordService;

    @Autowired
    ITaCustomerService iTaCustomerService;

    @Autowired
    ITaCarService iTaCarService;
    
    @Autowired
    ICameraService cameraService;

    @Autowired
    IMessageService iMessageService;

    @Override
    public boolean updateNormal(List<TaFirstUsherRecord> records) {
        if (records == null || records.size() == 0) return true;

        List<Integer> recs = new ArrayList<>();
        for (TaFirstUsherRecord rec: records) {
            recs.add(rec.getRecId());
        }

        UpdateWrapper<TaFirstUsherRecord> wrapper = new UpdateWrapper();
        wrapper.set("status", StatusUtils.Normal);
        wrapper.in("rec_id", recs);

        int row = taFirstUsherRecordMapper.update(new TaFirstUsherRecord(), wrapper);
        return row >= 0;
    }

    @Override
    public TaFirstUsherRecord newRecordByDevice(TaCarRecord taCarRecord, TaCustomer taCustomer) {
        TaFirstUsherRecord taFirstUsherRecord = new TaFirstUsherRecord();
        taFirstUsherRecord.setStatus(StatusUtils.Ready);
        taFirstUsherRecord.setCarImage(taCarRecord.getCarImage());
        taFirstUsherRecord.setCarModel(taCarRecord.getCarModel());
        taFirstUsherRecord.setColor(taCarRecord.getColor());
        taFirstUsherRecord.setCarRecord(taCarRecord.getSerialNo());
        taFirstUsherRecord.setPlateNumber(taCarRecord.getPlateNumber());
        taFirstUsherRecord.setVisiteDate(LocalDateTime.now());
        taFirstUsherRecord.setPersonName(taCarRecord.getCustomerName());
        taFirstUsherRecord.setPersonNum(taCarRecord.getPersonNum());
        taFirstUsherRecord.setDriverImage(taCarRecord.getDriverImage());
        taFirstUsherRecord.setLicenseImage(taCarRecord.getLicenseImage());
        if (null != taCustomer){
            taFirstUsherRecord.setConsultantId(taCustomer.getConsultantId());
            taFirstUsherRecord.setRealtyConsultant(taCustomer.getRealtyConsultant());
            taFirstUsherRecord.setPhone(taCustomer.getPhone());
            taFirstUsherRecord.setPersonName(taCustomer.getCustomerName());
        }
        
        taFirstUsherRecordMapper.insert(taFirstUsherRecord);

        return taFirstUsherRecord;
    }

    @Override
    public void notifyMiniapp(TaFirstUsherRecord taFirstUsherRecord) {
        Action act = Action.create(Action.TYPE_NEW_FIRST_USHER, JSONObject.toJSONString(taFirstUsherRecord, SerializerFeature.WriteMapNullValue));
        try {
            WebSocketServer.sendInfo(act.toJSON(), CommConstant.WECHAT_CLIENT_MINIAPP, null);
        } catch (Exception e) {}
    }
    
    @Override
    public void notifyCar(TaFirstUsherRecord taFirstUsherRecord) {
        Action act = Action.create(Action.TYPE_NEW_FIRST_USHER, JSONObject.toJSONString(taFirstUsherRecord, SerializerFeature.WriteMapNullValue));
        try {
            WebSocketServer.sendInfo(act.toJSON(), CommConstant.WECHAT_CLIENT_MINIAPP, null);
        } catch (Exception e) {}
    }

    @Override
    public boolean newRecordByMiniapp(TaFirstUsherRecord taFirstUsherRecord) {
        notifyConsultant(taFirstUsherRecord);
        int row = taFirstUsherRecordMapper.insert(taFirstUsherRecord);

        updateOtherInfo(taFirstUsherRecord);

        return row > 0;
    }

    @Override
    public boolean updateByMiniapp(TaFirstUsherRecord taFirstUsherRecord) {
        notifyConsultant(taFirstUsherRecord);
        int row = taFirstUsherRecordMapper.updateById(taFirstUsherRecord);

        updateOtherInfo(taFirstUsherRecord);

        return row > 0;
    }

    // 反更新人员表, 客户表, 来访表-今天
    private boolean updateOtherInfo(TaFirstUsherRecord taFirstUsherRecord) {
        // 更新车辆信息
        TaCar taCar = iTaCarService.getById(taFirstUsherRecord.getPlateNumber());
        if (null == taCar) {
            taCar = new TaCar();
            taCar.setPlateNumber(taFirstUsherRecord.getPlateNumber());
            taCar.setColor(taFirstUsherRecord.getColor());
            taCar.setCarModel(taFirstUsherRecord.getCarModel());
            taCar.setCarImage(taFirstUsherRecord.getCarImage());
            taCar.setStatus(StatusUtils.Normal);
            taCar.setTypeId(CommConstant.TYPE_CAR_NORMAL);
            iTaCarService.save(taCar);
        } else if (null != taFirstUsherRecord.getCarModel()) {
            taCar.setCarModel(taFirstUsherRecord.getCarModel());
            taCar.setColor(taFirstUsherRecord.getColor());
            iTaCarService.updateById(taCar);
        }

        // 查找客户
        TaCustomer taCustomer = iTaCustomerService.getByPhoneOrPlate(taFirstUsherRecord.getPhone(), taFirstUsherRecord.getPlateNumber());
        if (null == taCustomer) {
            if (!StringUtils.isEmpty(taFirstUsherRecord.getPhone())){
                try {
                    taCustomer = iTaCustomerService.newCustByFirstUsher(taFirstUsherRecord,true);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }else {
                return true;
            }
        }

        // 更新车型
        if (null != taFirstUsherRecord.getCarModel()) {
            taCustomer.setCarModel(taFirstUsherRecord.getCarModel());
            taCustomer.setCarColor(taFirstUsherRecord.getColor());
        }

        // 更新车牌
        if (StringUtils.isEmpty(taCustomer.getPlateNumber())) {
            taCustomer.setPlateNumber(taFirstUsherRecord.getPlateNumber());
        }

        iTaCustomerService.updateById(taCustomer);

        // 更新到访记录
        TaMainUsherRecord taMainUsherRecord = iTaMainUsherRecordService.getTodayRecord(taCustomer.getCustomerId());
        if (null == taMainUsherRecord) {
            iTaMainUsherRecordService.newRecordByFirstUsher(taFirstUsherRecord, taCustomer);
        } else {

            if (null != taFirstUsherRecord.getCarModel()) {
                taMainUsherRecord.setCarModel(taFirstUsherRecord.getCarModel());
                taMainUsherRecord.setColor(taFirstUsherRecord.getColor());
            }

            if (StringUtils.isEmpty(taMainUsherRecord.getPlateNumber())) {
                taMainUsherRecord.setPlateNumber(taFirstUsherRecord.getPlateNumber());
            }
            
            if(null != taMainUsherRecord.getPersonNum()){
                taMainUsherRecord.setPersonNum(taFirstUsherRecord.getPersonNum());
            }

            iTaMainUsherRecordService.updateById(taMainUsherRecord);
        }

        return true;
    }

    @Override
    public boolean removeRecordByFront(Integer id) {
        UpdateWrapper<TaFirstUsherRecord> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("status", StatusUtils.Delete);
        updateWrapper.eq("rec_id", id);

        int row = taFirstUsherRecordMapper.update(new TaFirstUsherRecord(), updateWrapper);
        return row > 0;
    }

    @Override
    public boolean addCar2WhiteList(Integer id) {
        UpdateWrapper<TaFirstUsherRecord> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("status", StatusUtils.Normal);
        updateWrapper.eq("rec_id", id);

        int row = taFirstUsherRecordMapper.update(new TaFirstUsherRecord(), updateWrapper);
        if (row < 1) {
            return false;
        }

        // 更新车牌为内部车辆
        TaFirstUsherRecord rec = taFirstUsherRecordMapper.selectById(id);
        if (null != rec && null != rec.getPlateNumber()) {
            iTaCarService.setCarType(rec.getPlateNumber(), CommConstant.TYPE_CAR_INNER);
        }

        return row > 0;
    }

    private void notifyConsultant(TaFirstUsherRecord taFirstUsherRecord) {
        String plateNumber = taFirstUsherRecord.getPlateNumber();
        String phone = taFirstUsherRecord.getPhone();

        TaCustomer taCustomer = iTaCustomerService.getByPhoneOrPlate(phone, plateNumber);
        if (null != taCustomer) {
            try {
                //通知前端
                iTaCustomerService.sendMessageAll(taFirstUsherRecord,taCustomer,null);
                //通知置业经理车辆信息
//                iTaCustomerService.notifyConsultant(taCustomer);
                iMessageService.toggledByCustomer(taCustomer);
                // 通知吧台
//                iTaCustomerService.sendMessageToBar(taCustomer, taFirstUsherRecord.getPersonNum());
            } catch (Exception e) {}
        }else if(StringUtils.isEmpty(taFirstUsherRecord.getPhone())){
            //有手机号的情况下会新建customer，推送后置
            //通知置业经理车辆信息
//            iTaCustomerService.notifyFirstUsherConsultantManager(taFirstUsherRecord, taCustomer);
            //通知前端
            iTaCustomerService.sendMessageAll(taFirstUsherRecord,taCustomer,null);
        }
    }

    @Override
    public IPage<TaFirstUsherRecord> taFirstUsherRecordlist(IPage pg,String plateNumber, String visiteDate) {
        IPage<TaFirstUsherRecord> pageTaFirstUsherRecord= taFirstUsherRecordMapper.taFirstUsherRecordlist(pg,plateNumber,visiteDate);
        return pageTaFirstUsherRecord;
    }
}
